package logicland2;

import logicland2.block.Block;
import logicland2.block.Empty;

public class Chunk {

	public static int size = 128;
	
	private Block[] blockList = new Block[size * size];
	private Pos pos;
	
	public Chunk(Pos inputPos) {
		pos = inputPos;
		int index = 0;
		while (index < blockList.length) {
			blockList[index] = null;
			index += 1;
		}
	}
	
	public Pos getPos() {
		return pos;
	}
	
	// Returns -1 if the position is out of bounds.
	private int convertPosToIndex(Pos inputPos) {
		int tempOffsetX = inputPos.getX() - pos.getX();
		int tempOffsetY = inputPos.getY() - pos.getY();
		if (tempOffsetX < 0 || tempOffsetX >= size || tempOffsetY < 0 || tempOffsetY >= size) {
			return -1;
		}
		return tempOffsetX + size * tempOffsetY;
	}
	
	public Block getBlock(Pos inputPos) {
		int index = convertPosToIndex(inputPos);
		if (index < 0) {
			return null;
		}
		return blockList[index];
	}
	
	public void setBlock(Pos inputPos, Block block) {
		int index = convertPosToIndex(inputPos);
		if (index < 0) {
			return;
		}
		blockList[index] = block;
	}
	
	public boolean isEmpty() {
		int index = 0;
		while (index < blockList.length) {
			Block tempBlock = blockList[index];
			if (tempBlock != null) {
				if (!(tempBlock instanceof Empty)) {
					return false;
				}
			}
			index += 1;
		}
		return true;
	}
	
	public boolean rowIsEmpty(int offsetY) {
		Pos tempPos = pos.copy();
		tempPos.setY(tempPos.getY() + offsetY);
		int tempOffsetX = 0;
		while (tempOffsetX < size) {
			Block tempBlock = getBlock(tempPos);
			if (tempBlock != null) {
				if (!(tempBlock instanceof Empty)) {
					return false;
				}
			}
			tempPos.setX(tempPos.getX() + 1);
			tempOffsetX += 1;
		}
		return true;
	}
	
}
