package logicland2;

import logicland2.block.Block;
import logicland2.block.Empty;

public class Clipboard {

	private int width;
	private int height;
	private Block[] blockList = null;
	private Pos marker1Offset = new Pos(0, 0);
	private Empty empty = new Empty();
	
	public Clipboard() {
		setDimensions(1, 1);
	}
	
	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
	public void setDimensions(int inputWidth, int inputHeight) {
		width = inputWidth;
		height = inputHeight;
		blockList = new Block[width * height];
		int index = 0;
		while (index < blockList.length) {
			blockList[index] = null;
			index += 1;
		}
	}
	
	public Pos getMarker1Offset() {
		return marker1Offset;
	}
	
	public void setMarker1Offset(Pos offset) {
		marker1Offset = offset;
	}
	
	private int convertPosToIndex(Pos pos) {
		return pos.getX() + pos.getY() * width;
	}
	
	public Block getBlock(Pos pos) {
		int index = convertPosToIndex(pos);
		Block output = blockList[index];
		if (output == null) {
			output = empty;
		}
		return output;
	}
	
	public void setBlock(Pos pos, Block block) {
		int index = convertPosToIndex(pos);
		if (block != null) {
			if (block instanceof Empty) {
				block = null;
			}
		}
		blockList[index] = block;
	}
	
	public void rotateClockwise() {
		int tempOldWidth = width;
		int tempOldHeight = height;
		Block[] tempOldBlockList = blockList;
		setDimensions(height, width);
		Pos tempPos = new Pos(0, 0);
		Pos tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < tempOldHeight) {
			tempPos.setX(tempOldHeight - tempOffset.getY() - 1);
			tempPos.setY(tempOffset.getX());
			int tempIndex1 = tempOffset.getX() + tempOffset.getY() * tempOldWidth;
			int tempIndex2 = tempPos.getX() + tempPos.getY() * width;
			Block tempBlock = tempOldBlockList[tempIndex1];
			if (tempBlock != null) {
				tempBlock.rotateClockwise();
			}
			blockList[tempIndex2] = tempBlock;
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= tempOldWidth) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
	}
	
	public void mirrorHorizontally() {
		Block[] tempOldBlockList = blockList;
		blockList = new Block[tempOldBlockList.length];
		Pos tempPos = new Pos(0, 0);
		Pos tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < height) {
			tempPos.setX(width - tempOffset.getX() - 1);
			tempPos.setY(tempOffset.getY());
			int tempIndex1 = tempOffset.getX() + tempOffset.getY() * width;
			int tempIndex2 = tempPos.getX() + tempPos.getY() * width;
			Block tempBlock = tempOldBlockList[tempIndex1];
			if (tempBlock != null) {
				tempBlock.mirrorHorizontally();
			}
			blockList[tempIndex2] = tempBlock;
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= width) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
	}
	
}
