package logicland2;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import logicland2.block.Block;
import logicland2.block.BufferLogicInput;
import logicland2.block.BufferLogicOutput;
import logicland2.block.Empty;
import logicland2.block.OrLogicModule;
import logicland2.block.SwitchInputPeripheral;
import logicland2.block.Wire;

public class CustomPanel extends JPanel implements SaveChangeSubscriber {

	public static int worldViewSize = 512;
	private static int sidePanelWidth = 512;
	private static int width = worldViewSize + sidePanelWidth;
	private static int height = worldViewSize;
	private static Color worldBackgroundColor = new Color(128, 128, 128);
	private static Color sidePanelBackgroundColor = new Color(255, 255, 255);
	private static Color inactiveButtonColor = new Color(192, 192, 192);
	private static Color activeButtonColor = new Color(0, 192, 0);
	private static Color gridColor = new Color(96, 96, 96);
	private static Color selectionColor = new Color(0, 255, 255);
	private static Color textColor = new Color(0, 0, 0);
	private static Font textFont = new Font("Verdana", Font.PLAIN, 16);
	private Image spriteSheet1;
	private Image spriteSheet2;
	
	private World world;
	private JFrame frame;
	private double zoom = 32.0;
	private Pos cameraPos = new Pos(0, 0);
	private List<Button> buttonList = new ArrayList<Button>();
	private List<Button> paletteButtonList = new ArrayList<Button>();
	private List<Brushable> brushList = new ArrayList<Brushable>();
	private int selectedBrushIndex = 0;
	// 0 = Edit mode.
	// 1 = Interaction mode.
	private int mode = 0;
	private Button modeButton;
	private Button gridButton;
	private Button pauseButton;
	private Button pastePreviewButton;
	private boolean isShowingGrid = false;
	private boolean isShowingPastePreview = false;
	
	public CustomPanel(World inputWorld, JFrame inputFrame) {
		super();
		try {
			spriteSheet1 = ImageIO.read(getClass().getResource("sprites1.gif"));
			spriteSheet2 = ImageIO.read(getClass().getResource("sprites2.gif"));
		} catch (IOException e) {
			System.out.println("ERROR: Unable to open sprite sheet.");
		}
		world = inputWorld;
		frame = inputFrame;
		
		brushList.add(new Empty());
		brushList.add(new AutoWire());
		brushList.add(new Wire(0));
		brushList.add(new Wire(2));
		brushList.add(new Wire(6));
		brushList.add(new Wire(10));
		brushList.add(new Wire(11));
		brushList.add(new BufferLogicInput());
		brushList.add(new BufferLogicOutput());
		brushList.add(new OrLogicModule());
		brushList.add(new SwitchInputPeripheral());
		brushList.add(new AutoStraight(0));
		brushList.add(new Marker(1));
		brushList.add(new Marker(2));
		
		int buttonSpacing = 10;
		int tempButtonPosX = worldViewSize + buttonSpacing;
		int tempButtonPosY = buttonSpacing;
		int tempButtonWidth = sidePanelWidth / 2 - buttonSpacing * 2;
		Button tempButton;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "E: Empty", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(0);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "1: Auto Wire", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(1);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "2: Straight Wire", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(2);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "3: Bent Wire", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(3);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "4: Three Split Wire", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(4);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "5: Four Split Wire", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(5);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "6: Cross Over", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(6);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "7: Logic Input", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(7);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "8: Logic Output", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(8);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "9: Logic Module", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(9);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "0: Interactive Module", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(10);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "[: Previous Variation", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectPreviousVariation();
        });
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "]: Next Variation", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectNextVariation();
        });
		buttonList.add(tempButton);
		
		tempButtonPosX += sidePanelWidth / 2;
		tempButtonPosY = buttonSpacing;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "X: Auto Straight", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(11);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "+: Zoom In", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	zoomIn();
        });
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "-: Zoom Out", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	zoomOut();
        });
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "G: Toggle Grid", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	toggleGrid();
        });
		gridButton = tempButton;
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "Space: Stop Simulation", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	toggleIsPaused();
        });
		pauseButton = tempButton;
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "T: Tick Clock", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	world.advanceClockState();
        });
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), ">: Speed Up", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	world.speedUpClock();
        });
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "<: Slow Down", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	world.slowDownClock();
        });
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "J: First Marker", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(12);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "K: Second Marker", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	selectBrush(13);
        });
		buttonList.add(tempButton);
		paletteButtonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "P: Preview Paste", tempButtonWidth);
		tempButton.setActionListener(event -> {
			togglePastePreview();
        });
		pastePreviewButton = tempButton;
		buttonList.add(tempButton);
		tempButtonPosY += buttonSpacing + Button.height;
		tempButton = new Button(new Pos(tempButtonPosX, tempButtonPosY), "M: Interaction Mode", tempButtonWidth);
		tempButton.setActionListener(event -> {
        	toggleMode();
        });
		modeButton = tempButton;
		buttonList.add(tempButton);
		
		selectBrush(1);
		updateTitle();
	}
	
	public void paint(Graphics graphics) {
		graphics.setColor(worldBackgroundColor);
		graphics.fillRect(0, 0, worldViewSize, worldViewSize);
		
		graphics.setColor(gridColor);
		if (isShowingGrid && zoom >= 8.0) {
			int tempPos = (int)zoom;
			while (tempPos < worldViewSize) {
				graphics.fillRect(0, tempPos, worldViewSize, 1);
				graphics.fillRect(tempPos, 0, 1, worldViewSize);
				tempPos += zoom;
			}
		}
		
		int tempBlockCount = (int)(worldViewSize / zoom);
		int tempStride;
		if (zoom < 1.0) {
			tempStride = (int)(1 / zoom);
		} else {
			tempStride = 1;
		}
		Pos tempPos = new Pos(0, 0);
		Pos tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < tempBlockCount) {
			tempPos.set(tempOffset);
			tempPos.add(cameraPos);
			Block tempBlock = world.getBlock(tempPos);
			if (tempBlock != null) {
				tempBlock.draw(this, graphics, tempPos);
			}
			tempOffset.setX(tempOffset.getX() + tempStride);
			if (tempOffset.getX() >= tempBlockCount) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + tempStride);
			}
		}
		
		if (mode == 0) {
			tempPos = world.getBrushPos();
			if (tempPos != null) {
				Brushable tempBlock = world.getBrush();
				if (tempBlock != null) {
					if (!tempBlock.canOverlapBlock()) {
						Pos tempPos2 = tempPos.copy();
						tempPos2.subtract(cameraPos);
						tempPos2.scale(zoom);
						if (zoom >= 8.0) {
							graphics.setColor(worldBackgroundColor);
							graphics.fillRect(tempPos2.getX(), tempPos2.getY(), (int)zoom, (int)zoom);
							if (isShowingGrid) {
								graphics.setColor(gridColor);
								if (tempPos2.getY() > 0) {
									graphics.fillRect(tempPos2.getX(), tempPos2.getY(), (int)zoom, 1);
								}
								if (tempPos2.getX() > 0) {
									graphics.fillRect(tempPos2.getX(), tempPos2.getY(), 1, (int)zoom);
								}
							}
						}
					}
					tempBlock.draw(this, graphics, tempPos);
				}
			}
		}
		
		Marker tempMarker1 = world.getMarker(1);
		Marker tempMarker2 = world.getMarker(2);
		if (isShowingPastePreview) {
			if (tempMarker1 != null) {
				Clipboard tempClipboard = world.getClipboard();
				tempPos = new Pos(0, 0);
				tempOffset = new Pos(0, 0);
				while (tempOffset.getY() < tempClipboard.getHeight()) {
					tempPos = tempMarker1.getPos().copy();
					tempPos.subtract(tempClipboard.getMarker1Offset());
					tempPos.add(tempOffset);
					if (worldPosIsInView(tempPos)) {
						Block tempBlock = tempClipboard.getBlock(tempOffset);
						tempBlock.draw(this, graphics, tempPos);
					}
					tempOffset.setX(tempOffset.getX() + 1);
					if (tempOffset.getX() >= tempClipboard.getWidth()) {
						tempOffset.setX(0);
						tempOffset.setY(tempOffset.getY() + 1);
					}
				}
				tempPos = tempMarker1.getPos().copy();
				tempPos.subtract(tempClipboard.getMarker1Offset());
				tempPos.subtract(cameraPos);
				tempPos.scale(zoom);
				int tempWidth = (int)(tempClipboard.getWidth() * zoom - 1);
				int tempHeight = (int)(tempClipboard.getHeight() * zoom - 1);
				graphics.setColor(selectionColor);
				graphics.fillRect(tempPos.getX(), tempPos.getY(), tempWidth, 1);
				graphics.fillRect(tempPos.getX(), tempPos.getY() + tempHeight, tempWidth, 1);
				graphics.fillRect(tempPos.getX(), tempPos.getY(), 1, tempHeight);
				graphics.fillRect(tempPos.getX() + tempWidth, tempPos.getY(), 1, tempHeight);
			}
		} else {
			if (world.hasSelection()) {
				Pos tempPos1 = world.getSelectionTopLeft();
				Pos tempPos2 = world.getSelectionBottomRight();
				tempPos1.subtract(cameraPos);
				tempPos1.scale(zoom);
				tempPos2.subtract(cameraPos);
				tempPos2.scale(zoom);
				int tempPosX1 = tempPos1.getX();
				int tempPosX2 = tempPos2.getX() + (int)zoom - 1;
				int tempPosY1 = tempPos1.getY();
				int tempPosY2 = tempPos2.getY() + (int)zoom - 1;
				graphics.setColor(selectionColor);
				graphics.fillRect(tempPosX1, tempPosY1, tempPosX2 - tempPosX1, 1);
				graphics.fillRect(tempPosX1, tempPosY2, tempPosX2 - tempPosX1, 1);
				graphics.fillRect(tempPosX1, tempPosY1, 1, tempPosY2 - tempPosY1);
				graphics.fillRect(tempPosX2, tempPosY1, 1, tempPosY2 - tempPosY1);
			}
		}
		if (tempMarker1 != null) {
			tempMarker1.draw(this, graphics, tempMarker1.getPos());
		}
		if (tempMarker2 != null) {
			tempMarker2.draw(this, graphics, tempMarker2.getPos());
		}
		
		graphics.setColor(sidePanelBackgroundColor);
		graphics.fillRect(worldViewSize, 0, sidePanelWidth, worldViewSize);
		
		int index = 0;
		while (index < buttonList.size()) {
			Button tempButton = buttonList.get(index);
			tempButton.draw(graphics);
			index += 1;
		}
		
		/*
		graphics.setFont(textFont);
		graphics.setColor(textColor);
		graphics.drawString("For more controls, please", worldViewSize + sidePanelWidth / 2 + 16, worldViewSize - 40);
		graphics.drawString("consult the file menu.", worldViewSize + sidePanelWidth / 2 + 16, worldViewSize - 20);
		*/

	}
	
	public void drawSmallSprite(Graphics graphics, Pos pos, int which) {
		int tempPosX = (which % 16) * 8;
		int tempPosY = (which / 16) * 8;
		int tempSpriteSize = (int)zoom;
		if (tempSpriteSize < 1) {
			tempSpriteSize = 1;
		}
		graphics.drawImage(spriteSheet2, pos.getX(), pos.getY(), pos.getX() + tempSpriteSize, pos.getY() + tempSpriteSize,
				tempPosX, tempPosY, tempPosX + 8, tempPosY + 8, null);
	}
	
	public void drawLargeSprite(Graphics graphics, Pos pos, int which) {
		int tempPosX = (which % 16) * 16;
		int tempPosY = (which / 16) * 16;
		int tempSpriteSize = (int)zoom;
		if (tempSpriteSize < 1) {
			tempSpriteSize = 1;
		}
		graphics.drawImage(spriteSheet1, pos.getX(), pos.getY(), pos.getX() + tempSpriteSize, pos.getY() + tempSpriteSize,
				tempPosX, tempPosY, tempPosX + 16, tempPosY + 16, null);
	}
	
	public void drawSprite(Graphics graphics, Pos pos, int which) {
		if (zoom >= 16.0) {
			drawLargeSprite(graphics, pos, which);
		} else {
			drawSmallSprite(graphics, pos, which);
		}
	}

	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
	public World getWorld() {
		return world;
	}
	
	public double getZoom() {
		return zoom;
	}
	
	public void setZoom(double inputZoom) {
		zoom = inputZoom;
	}
	
	public void zoomOut() {
		if (zoom <= 0.25) {
			return;
		}
		int tempOffset = (int)(worldViewSize / zoom) / 2;
		cameraPos.setX(cameraPos.getX() - tempOffset);
		cameraPos.setY(cameraPos.getY() - tempOffset);
		zoom /= 2.0;
	}
	
	public void zoomIn() {
		if (zoom >= 64.0) {
			return;
		}
		int tempOffset = (int)(worldViewSize / zoom) / 4;
		cameraPos.setX(cameraPos.getX() + tempOffset);
		cameraPos.setY(cameraPos.getY() + tempOffset);
		zoom *= 2.0;
	}
	
	public void scroll(Pos pos) {
		Pos tempPos = pos.copy();
		tempPos.scale((worldViewSize / zoom) / 8.0);
		cameraPos.add(tempPos);
	}
	
	public Button getButtonAtPos(Pos pos) {
		int index = 0;
		while (index < buttonList.size()) {
			Button tempButton = buttonList.get(index);
			if (pos.getX() >= tempButton.getPos().getX() && pos.getX() < tempButton.getPos().getX() + tempButton.getWidth() && pos.getY() >= tempButton.getPos().getY() && pos.getY() < tempButton.getPos().getY() + tempButton.getHeight()) {
				return tempButton;
			}
			index += 1;
		}
		return null;
	}
	
	public boolean registerButtonClick(Pos pos) {
		Button tempButton = getButtonAtPos(pos);
		if (tempButton == null) {
			return false;
		}
		tempButton.press();
		return true;
	}
	
	public Pos getCameraPos() {
		return cameraPos;
	}
	
	public void selectBrush(int index) {
		Button tempButton;
		tempButton = paletteButtonList.get(selectedBrushIndex);
		tempButton.setBackgroundColor(inactiveButtonColor);
		selectedBrushIndex = index;
		world.setBrush(brushList.get(selectedBrushIndex));
		tempButton = paletteButtonList.get(selectedBrushIndex);
		tempButton.setBackgroundColor(activeButtonColor);
		if (mode != 0) {
			mode = 0;
			updateModeButton();
		}
	}
	
	public void selectNextVariation() {
		Brushable tempBrush = world.getBrush();
		if (tempBrush == null) {
			return;
		}
		tempBrush = tempBrush.getNextVariation();
		world.setBrush(tempBrush);
		brushList.set(selectedBrushIndex, tempBrush);
	}
	
	public void selectPreviousVariation() {
		Brushable tempBrush = world.getBrush();
		if (tempBrush == null) {
			return;
		}
		tempBrush = tempBrush.getPreviousVariation();
		world.setBrush(tempBrush);
		brushList.set(selectedBrushIndex, tempBrush);
	}
	
	public int getMode() {
		return mode;
	}
	
	public void updateModeButton() {
		if (mode == 1) {
			modeButton.setBackgroundColor(activeButtonColor);
		} else {
			modeButton.setBackgroundColor(inactiveButtonColor);
		}
	}
	
	public void toggleMode() {
		mode += 1;
		if (mode > 1) {
			mode = 0;
		}
		updateModeButton();
	}
	
	public void toggleGrid() {
		isShowingGrid = !isShowingGrid;
		if (isShowingGrid) {
			gridButton.setBackgroundColor(activeButtonColor);
		} else {
			gridButton.setBackgroundColor(inactiveButtonColor);
		}
	}
	
	public void toggleIsPaused() {
		world.toggleIsPaused();
		if (world.getIsPaused()) {
			pauseButton.setBackgroundColor(activeButtonColor);
		} else {
			pauseButton.setBackgroundColor(inactiveButtonColor);
		}
	}
	
	public void togglePastePreview() {
		isShowingPastePreview = !isShowingPastePreview;
		if (isShowingPastePreview) {
			pastePreviewButton.setBackgroundColor(activeButtonColor);
		} else {
			pastePreviewButton.setBackgroundColor(inactiveButtonColor);
		}
	}
	
	public boolean worldPosIsInView(Pos pos) {
		Pos tempPos = pos.copy();
		tempPos.subtract(cameraPos);
		tempPos.scale(zoom);
		return (tempPos.getX() >= 0 && tempPos.getX() < worldViewSize && tempPos.getY() >= 0 && tempPos.getY() < worldViewSize);
	}
	
	private String getFileNameExtension(String fileName) {
		int index = fileName.lastIndexOf('.');
		if (index == -1) {
			return "";
		}
		return fileName.substring(index + 1, fileName.length());
	}
	
	public void saveAs() {
		JFileChooser tempFileChooser = new JFileChooser();
		FileFilter tempFilter = new FileNameExtensionFilter("Logic Land 2 File (*.LL2)", "LL2");
		tempFileChooser.setFileFilter(tempFilter);
		int tempValue = tempFileChooser.showSaveDialog(this);
		if (tempValue == JFileChooser.APPROVE_OPTION) {
			File tempFile = tempFileChooser.getSelectedFile();
			if (!getFileNameExtension(tempFile.getName()).equalsIgnoreCase("LL2")) {
				tempFile = new File(tempFile.getAbsolutePath() + ".LL2");
			}
			world.setFile(tempFile);
			world.saveToFile();
		}
	}
	
	public void save() {
		if (world.getFile() == null) {
			saveAs();
		} else {
			world.saveToFile();
		}
	}
	
	public void open() {
		confirmSave();
		JFileChooser tempFileChooser = new JFileChooser();
		FileFilter tempFilter = new FileNameExtensionFilter("Logic Land 2 File (*.LL2)", "LL2");
		tempFileChooser.setFileFilter(tempFilter);
		int tempValue = tempFileChooser.showOpenDialog(this);
		if (tempValue == JFileChooser.APPROVE_OPTION) {
			world.setFile(tempFileChooser.getSelectedFile());
			world.openFromFile();
		}
	}
	
	public void confirmSave() {
		if (!world.getSaveHasBeenChanged()) {
			return;
		}
		int dialogButton = JOptionPane.YES_NO_OPTION;
		int tempResult = JOptionPane.showConfirmDialog(null, "Would you like to save your work?", "Warning", dialogButton);
		if (tempResult == JOptionPane.YES_OPTION) {
			save();
		}
	}
	
	public void updateTitle() {
		String tempTitle;
		if (world.getSaveHasBeenChanged()) {
			tempTitle = "LOGIC LAND 2 (Unsaved Work)";
		} else {
			tempTitle = "LOGIC LAND 2";
		}
		frame.setTitle(tempTitle);
	}

	@Override
	public void saveChangeEvent() {
		updateTitle();
	}
	
}
