package logicland2;

import java.awt.Cursor;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.Timer;

public class Main {

	private static JFrame frame;
	private static CustomPanel panel;
	private static JMenuBar menuBar;
	private static Insets insets;
	private static boolean shiftKeyIsHeld = false;
	private static boolean controlKeyIsHeld = false;
	private static World world;
	
	public static void main(String[] args) {
		world = new World();
		frame = new JFrame();
		panel = new CustomPanel(world, frame);
		world.addSaveChangeSubscriber(panel);
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

		menuBar = new JMenuBar();
		JMenu menu;
		JMenuItem menuItem;
		menu = new JMenu("File");
        menuItem = new JMenuItem("(Ctrl+N) New Circuit");
        menuItem.addActionListener(event -> {
        	world.clearMapAndFileAssociation();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+O) Open...");
        menuItem.addActionListener(event -> {
        	panel.open();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+S) Save");
        menuItem.addActionListener(event -> {
        	panel.save();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("Save As...");
        menuItem.addActionListener(event -> {
        	panel.saveAs();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+Q) Quit");
        menuItem.addActionListener(event -> {
        	safelyQuit();
        });
        menu.add(menuItem);
        menuBar.add(menu);
        menu = new JMenu("Edit");
        menuItem = new JMenuItem("(Ctrl+Z) Undo");
        menuItem.addActionListener(event -> {
        	world.undoUserChange();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+Y) Redo");
        menuItem.addActionListener(event -> {
        	world.redoUserChange();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+C) Copy In Markers");
        menuItem.addActionListener(event -> {
        	world.copySelectionToClipboard();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+X) Cut");
        menuItem.addActionListener(event -> {
        	world.cutSelectionToClipboard();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+V) Paste");
        menuItem.addActionListener(event -> {
        	world.pasteClipboard();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+R) Rotate Clipboard");
        menuItem.addActionListener(event -> {
        	world.rotateClipboardClockwise();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+M) Mirror Clipboard");
        menuItem.addActionListener(event -> {
        	world.mirrorClipboardHorizontally();
        });
        menu.add(menuItem);
        menuItem = new JMenuItem("(Ctrl+F) Fill");
        menuItem.addActionListener(event -> {
        	world.fillSelection();
        });
        menu.add(menuItem);
        menuBar.add(menu);
        frame.setJMenuBar(menuBar);
		
		frame.pack();
		insets = frame.getInsets();
		frame.setSize(insets.left + insets.right + panel.getWidth(),
				insets.top + insets.bottom + menuBar.getHeight() + panel.getHeight());
		frame.setContentPane(panel);
        
		frame.setVisible(true);
		ActionListener actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				world.tick();
				panel.repaint();
			}
		};
		Timer timer = new Timer(50, actionListener);
		timer.start();
		KeyListener keyListener = new KeyListener() {
			public void keyPressed(KeyEvent actionEvent) {
				int keyCode = actionEvent.getKeyCode();
				if (controlKeyIsHeld) {
					// Control + N.
					if (keyCode == 78) {
						world.clearMapAndFileAssociation();
					}
					// O.
					if (keyCode == 79) {
						panel.open();
					}
					// Control + S
					if (keyCode == 83 && !shiftKeyIsHeld) {
						panel.save();
					}
					// Control + Shift + S
					if (keyCode == 83 && shiftKeyIsHeld) {
						panel.saveAs();
					}
					// Q.
					if (keyCode == 81) {
						safelyQuit();
					}
					// Control + Z.
					if (keyCode == 90) {
						world.undoUserChange();
					}
					// Control + Y.
					if (keyCode == 89) {
						world.redoUserChange();
					}
					// Control + C.
					if (keyCode == 67) {
						world.copySelectionToClipboard();
					}
					// Control + X.
					if (keyCode == 88) {
						world.cutSelectionToClipboard();
					}
					// Control + V.
					if (keyCode == 86) {
						world.pasteClipboard();
					}
					// Control + R.
					if (keyCode == 82) {
						world.rotateClipboardClockwise();
					}
					// Control + M.
					if (keyCode == 77) {
						world.mirrorClipboardHorizontally();
					}
					// Control + F.
					if (keyCode == 70) {
						world.fillSelection();
					}
				} else {
					// Minus.
					if (keyCode == 45) {
						panel.zoomOut();
					}
					// Plus.
					if (keyCode == 61 && shiftKeyIsHeld) {
						panel.zoomIn();
					}
					// A.
					if (keyCode == 65 || keyCode == 37) {
						panel.scroll(new Pos(-1, 0));
					}
					// D.
					if (keyCode == 68 || keyCode == 39) {
						panel.scroll(new Pos(1, 0));
					}
					// W.
					if (keyCode == 87 || keyCode == 38) {
						panel.scroll(new Pos(0, -1));
					}
					// S.
					if (keyCode == 83 || keyCode == 40) {
						panel.scroll(new Pos(0, 1));
					}
					// [.
					if (keyCode == 91) {
						panel.selectPreviousVariation();
					}
					// ].
					if (keyCode == 93) {
						panel.selectNextVariation();
					}
					// E.
					if (keyCode == 69) {
						panel.selectBrush(0);
					}
					// 1 through 9.
					if (keyCode >= 49 && keyCode <= 57) {
						panel.selectBrush(keyCode - 48);
					}
					// 0.
					if (keyCode == 48) {
						panel.selectBrush(10);
					}
					// X.
					if (keyCode == 88) {
						panel.selectBrush(11);
					}
					// J.
					if (keyCode == 74) {
						panel.selectBrush(12);
					}
					// K.
					if (keyCode == 75) {
						panel.selectBrush(13);
					}
					// M.
					if (keyCode == 77) {
						panel.toggleMode();
					}
					// G.
					if (keyCode == 71) {
						panel.toggleGrid();
					}
					// >.
					if (keyCode == 46 && shiftKeyIsHeld) {
						world.speedUpClock();
					}
					// <.
					if (keyCode == 44 && shiftKeyIsHeld) {
						world.slowDownClock();
					}
					// Space.
					if (keyCode == 32) {
						panel.toggleIsPaused();
					}
					// T.
					if (keyCode == 84) {
						world.advanceClockState();
					}
					// P.
					if (keyCode == 80) {
						panel.togglePastePreview();
					}
				}
				// Shift.
				if (keyCode == 16) {
					shiftKeyIsHeld = true;
				}
				// Control.
				if (keyCode == 17 || keyCode == 157) {
					controlKeyIsHeld = true;
				}
			}
			public void keyReleased(KeyEvent actionEvent) {
				int keyCode = actionEvent.getKeyCode();
				
				// Shift.
				if (keyCode == 16) {
					shiftKeyIsHeld = false;
				}
				// Control.
				if (keyCode == 17 || keyCode == 157) {
					controlKeyIsHeld = false;
				}
			}
			public void keyTyped(KeyEvent actionEvent) {
				// Do nothing.
			}
		};
		frame.addKeyListener(keyListener);
		MouseListener mouseListener = new MouseListener() {
			@Override
			public void mouseClicked(MouseEvent e) {
				// Do nothing.
			}

			@Override
			public void mousePressed(MouseEvent e) {
				Pos tempPos = convertMouseEventToPos(e);
				world.setUserHasChangedBlock(false);
				if (tempPos.getX() < CustomPanel.worldViewSize) {
					Pos tempWorldPos = convertMousePosToWorldPos(tempPos);
					int tempMode = panel.getMode();
					if (tempMode == 0) {
						world.setBrushPos(tempWorldPos);
						world.paintBrush();
					}
					if (tempMode == 1) {
						world.interact(tempWorldPos);
					}
				} else {
					panel.registerButtonClick(tempPos);
				}
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				world.handleUserChanges();
				world.stopInteraction();
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				// Do nothing.
			}

			@Override
			public void mouseExited(MouseEvent e) {
				// Do nothing.
			}
		};
		frame.addMouseListener(mouseListener);
		MouseMotionListener mouseMotionListener = new MouseMotionListener() {

			@Override
			public void mouseDragged(MouseEvent e) {
				Pos tempPos = convertMouseEventToPos(e);
				if (tempPos.getX() < CustomPanel.worldViewSize) {
					Pos tempWorldPos = convertMousePosToWorldPos(tempPos);
					int tempMode = panel.getMode();
					if (tempMode == 0) {
						world.paintFromLastBrushPos(tempWorldPos);
					}
				}
			}

			@Override
			public void mouseMoved(MouseEvent e) {
				Pos tempPos = convertMouseEventToPos(e);
				if (tempPos.getX() < CustomPanel.worldViewSize) {
					Pos tempWorldPos = convertMousePosToWorldPos(tempPos);
					world.setBrushPos(tempWorldPos);
					panel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				} else {
					if (panel.getButtonAtPos(tempPos) == null){
						panel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			        } else {
			        	panel.setCursor(new Cursor(Cursor.HAND_CURSOR));
			        }
				}
			}
		};
		frame.addMouseMotionListener(mouseMotionListener);
		frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent e) {
                safelyQuit();
            }
        });
	}
	
	public static void safelyQuit() {
		panel.confirmSave();
		System.exit(0);
	}
	
	public static Pos convertMouseEventToPos(MouseEvent event) {
		return new Pos(event.getX() - insets.left, event.getY() - insets.top - menuBar.getHeight());
	}
	
	public static Pos convertMousePosToWorldPos(Pos pos) {
		Pos output = new Pos((int)(pos.getX() / panel.getZoom()), (int)(pos.getY() / panel.getZoom()));
		output.add(panel.getCameraPos());
		return output;
	}
	
}
