package logicland2;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import logicland2.block.AdditionLogicModule;
import logicland2.block.AndLogicModule;
import logicland2.block.Block;
import logicland2.block.BufferLogicInput;
import logicland2.block.BufferLogicOutput;
import logicland2.block.ButtonInputPeripheral;
import logicland2.block.CarryLogicOutput;
import logicland2.block.ClockInputPeripheral;
import logicland2.block.Empty;
import logicland2.block.ExtensionInputPeripheral;
import logicland2.block.ExtensionLogicModule;
import logicland2.block.ExtensionOutputPeripheral;
import logicland2.block.FlipFlopLogicModule;
import logicland2.block.InputPeripheral;
import logicland2.block.InvertLogicInput;
import logicland2.block.InvertLogicOutput;
import logicland2.block.LightOutputPeripheral;
import logicland2.block.ModuleConnectableBlock;
import logicland2.block.OrLogicModule;
import logicland2.block.ResetLogicInput;
import logicland2.block.SetLogicInput;
import logicland2.block.SwitchInputPeripheral;
import logicland2.block.TickableBlock;
import logicland2.block.ToggleLogicInput;
import logicland2.block.Wire;
import logicland2.block.WriteLogicInput;
import logicland2.block.XorLogicModule;

public class World {

	public static int saveCodePosCommand = 100;
	
	private Map<Pos, Chunk> chunkMap = new HashMap<Pos, Chunk>();
	private List<Chunk> chunkList = new ArrayList<Chunk>();
	private Brushable brush = new Wire(0);
	private Pos brushPos = null;
	private Empty empty = new Empty();
	private Pos interactionPos = null;
	private List<Signal> signalList = new ArrayList<Signal>();
	private List<TickableBlock> tickableBlockList = new ArrayList<TickableBlock>();
	private boolean clockState = false;
	private double clockDelay = 0.0;
	private double maximumClockDelay = 8.0;
	private boolean isPaused = false;
	private Marker marker1 = null;
	private Marker marker2 = null;
	private Clipboard clipboard = new Clipboard();
	private List<Block> saveCodeBlockList = new ArrayList<Block>();
	private boolean userHasChangedBlock = false;
	private List<String> changeHistory = new ArrayList<String>();
	private int changeHistoryIndex = -1;
	private File file = null;
	private boolean saveHasBeenChanged = false;
	private List<SaveChangeSubscriber> saveChangeSubscriberList = new ArrayList<SaveChangeSubscriber>();
	
	public World() {
		saveCodeBlockList.add(new Empty());
		saveCodeBlockList.add(new Wire(0));
		saveCodeBlockList.add(new BufferLogicInput());
		saveCodeBlockList.add(new InvertLogicInput());
		saveCodeBlockList.add(new SetLogicInput());
		saveCodeBlockList.add(new ResetLogicInput());
		saveCodeBlockList.add(new WriteLogicInput());
		saveCodeBlockList.add(new ToggleLogicInput());
		saveCodeBlockList.add(new BufferLogicOutput());
		saveCodeBlockList.add(new InvertLogicOutput());
		saveCodeBlockList.add(new CarryLogicOutput());
		saveCodeBlockList.add(new OrLogicModule());
		saveCodeBlockList.add(new AndLogicModule());
		saveCodeBlockList.add(new XorLogicModule());
		saveCodeBlockList.add(new FlipFlopLogicModule());
		saveCodeBlockList.add(new AdditionLogicModule());
		saveCodeBlockList.add(new ExtensionLogicModule());
		saveCodeBlockList.add(new SwitchInputPeripheral());
		saveCodeBlockList.add(new ButtonInputPeripheral());
		saveCodeBlockList.add(new ClockInputPeripheral());
		saveCodeBlockList.add(new ExtensionInputPeripheral());
		saveCodeBlockList.add(new LightOutputPeripheral());
		saveCodeBlockList.add(new ExtensionOutputPeripheral());
		
		clearChangeHistory();
		clearMapAndFileAssociation();
		setSaveHasBeenChanged(false);
	}
	
	public void clearMap() {
		chunkMap = new HashMap<Pos, Chunk>();
		chunkList = new ArrayList<Chunk>();
		signalList = new ArrayList<Signal>();
		tickableBlockList = new ArrayList<TickableBlock>();
		marker1 = null;
		marker2 = null;
	}
	
	public void clearMapAndFileAssociation() {
		clearMap();
		file = null;
		userHasChangedBlock = true;
		handleUserChanges();
	}
	
	public void clearChangeHistory() {
		changeHistory = new ArrayList<String>();
		changeHistoryIndex = -1;
	}
	
	private Pos convertPosToChunkPos(Pos pos) {
		return new Pos((int)(Math.floor(1.0 * pos.getX() / Chunk.size) * Chunk.size), (int)(Math.floor(1.0 * pos.getY() / Chunk.size) * Chunk.size));
	}
	
	public void addChunk(Chunk chunk) {
		chunkMap.put(chunk.getPos(), chunk);
		chunkList.add(chunk);
	}
	
	public Block getBlock(Pos pos) {
		Pos tempPos = convertPosToChunkPos(pos);
		Chunk tempChunk;
		if (!chunkMap.containsKey(tempPos)) {
			tempChunk = new Chunk(tempPos);
			addChunk(tempChunk);
		} else {
			tempChunk = chunkMap.get(tempPos);
		}
		Block output = tempChunk.getBlock(pos);
		if (output == null) {
			return empty;
		}
		return output;
	}
	
	public void setBlock(Pos pos, Block block) {
		Block tempBlock = getBlock(pos);
		Block tempBlock2 = block;
		if (block != null) {
			if (block instanceof Empty) {
				block = null;
			}
		}
		Pos tempPos = convertPosToChunkPos(pos);
		Chunk tempChunk;
		if (!chunkMap.containsKey(tempPos)) {
			tempChunk = new Chunk(tempPos);
			addChunk(tempChunk);
		} else {
			tempChunk = chunkMap.get(tempPos);
		}
		tempChunk.setBlock(pos, null);
		tempBlock.removeEvent(this, pos);
		tempChunk.setBlock(pos, block);
		tempBlock2.addEvent(this, pos);
	}
	
	public Pos getBrushPos() {
		return brushPos;
	}
	
	public void setBrushPos(Pos pos) {
		brushPos = pos;
	}
	
	public Brushable getBrush() {
		return brush;
	}
	
	public void setBrush(Brushable inputBrush) {
		brush = inputBrush;
	}
	
	public void paintBrush() {
		if (brush.addToWorld(this, brushPos)) {
			userHasChangedBlock = true;
		}
	}
	
	public void paintFromLastBrushPos(Pos pos) {
		while (brushPos.getX() < pos.getX()) {
			paintBrush();
			brushPos.setX(brushPos.getX() + 1);
		}
		while (brushPos.getX() > pos.getX()) {
			paintBrush();
			brushPos.setX(brushPos.getX() - 1);
		}
		while (brushPos.getY() < pos.getY()) {
			paintBrush();
			brushPos.setY(brushPos.getY() + 1);
		}
		while (brushPos.getY() > pos.getY()) {
			paintBrush();
			brushPos.setY(brushPos.getY() - 1);
		}
		paintBrush();
	}
	
	public static Pos getPosOnSide(Pos pos, int side) {
		Pos output = pos.copy();
		if (side == 0) {
			output.setY(output.getY() - 1);
		}
		if (side == 1) {
			output.setX(output.getX() + 1);
		}
		if (side == 2) {
			output.setY(output.getY() + 1);
		}
		if (side == 3) {
			output.setX(output.getX() - 1);
		}
		return output;
	}
	
	public boolean updateWireConnections(Pos pos) {
		Block tempBlock = getBlock(pos);
		if (!(tempBlock instanceof Wire)) {
			return false;
		}
		Wire tempWire = (Wire)tempBlock;
		int tempDirection1 = tempWire.getDirection();
		int tempSide = 0;
		while (tempSide <= 3) {
			Block tempBlock2 = getBlock(getPosOnSide(pos, tempSide));
			if (tempBlock2.isWireConnectable()) {
				tempWire.connectOnSide(tempSide);
			}
			tempSide += 1;
		}
		tempSide = 0;
		while (tempSide <= 3) {
			Block tempBlock2 = getBlock(getPosOnSide(pos, tempSide));
			if (!tempBlock2.isWireConnectable()) {
				tempWire.disconnectOnSide(tempSide);
			}
			tempSide += 1;
		}
		int tempDirection2 = tempWire.getDirection();
		return (tempDirection1 != tempDirection2);
	}
	
	public void interact(Pos pos) {
		Block tempBlock = getBlock(pos);
		if (tempBlock != null) {
			if (tempBlock instanceof InputPeripheral) {
				InputPeripheral tempInputPeripheral = (InputPeripheral)tempBlock;
				tempInputPeripheral.interact();
				interactionPos = pos.copy();
			}
		}
	}
	
	public void stopInteraction() {
		if (interactionPos == null) {
			return;
		}
		Block tempBlock = getBlock(interactionPos);
		if (tempBlock != null) {
			if (tempBlock instanceof InputPeripheral) {
				InputPeripheral tempInputPeripheral = (InputPeripheral)tempBlock;
				tempInputPeripheral.stopInteraction();
			}
		}
	}
	
	public void propagateModuleConnectionUpdates(Pos pos) {
		List<Pos> tempPosList = new ArrayList<Pos>();
		tempPosList.add(pos);
		while (tempPosList.size() > 0) {
			List<Pos> tempPosList2 = new ArrayList<Pos>();
			int index = 0;
			while (index < tempPosList.size()) {
				Pos tempPos = tempPosList.get(index);
				Block tempBlock = getBlock(tempPos);
				if (tempBlock instanceof ModuleConnectableBlock) {
					ModuleConnectableBlock tempConnectableBlock = (ModuleConnectableBlock)tempBlock;
					if (tempConnectableBlock.updateConnection(this, tempPos)) {
						int tempSide = 0;
						while (tempSide <= 3) {
							tempPosList2.add(getPosOnSide(tempPos, tempSide));
							tempSide += 1;
						}
					}
				}
				index += 1;
			}
			tempPosList = tempPosList2;
		}
	}
	
	public void addSignal(Signal signal) {
		signalList.add(signal);
	}
	
	public void propagateSignals() {
		int tempCount = 0;
		while (signalList.size() > 0 && tempCount < 100000) {
			List<Signal> tempOldSignalList = signalList;
			signalList = new ArrayList<Signal>();
			int index = 0;
			while (index < tempOldSignalList.size()) {
				Signal tempSignal = tempOldSignalList.get(index);
				Block tempBlock = getBlock(tempSignal.getPos());
				tempBlock.propagateSignal(this, tempSignal);
				index += 1;
			}
			tempCount += 1;
		}
	}
	
	public void advanceClockState() {
		clockState = !clockState;
		int index = 0;
		while (index < tickableBlockList.size()) {
			TickableBlock tempBlock = tickableBlockList.get(index);
			tempBlock.tick(this);
			index += 1;
		}
		propagateSignals();
	}
	
	public void tick() {
		long tempStartTime = System.currentTimeMillis();
		boolean hasPropagatedSignals = false;
		if (!isPaused) {
		clockDelay -= 1.0;
			while (clockDelay < 0.0) {
				advanceClockState();
				hasPropagatedSignals = true;
				clockDelay += maximumClockDelay;
				long tempEndTime = System.currentTimeMillis();
				if (tempEndTime - tempStartTime > 50) {
					clockDelay = 0.0;
					break;
				}
			}
		}
		if (!hasPropagatedSignals) {
			propagateSignals();
		}
	}
	
	public void registerTickableBlock(TickableBlock block) {
		if (!tickableBlockList.contains(block)) {
			tickableBlockList.add(block);
		}
	}
	
	public void deregisterTickableBlock(TickableBlock block) {
		if (tickableBlockList.contains(block)) {
			tickableBlockList.remove(block);
		}
	}
	
	public boolean getClockState() {
		return clockState;
	}
	
	public void speedUpClock() {
		if (maximumClockDelay > 0.001) {
			maximumClockDelay /= 2.0;
		}
	}
	
	public void slowDownClock() {
		if (maximumClockDelay < 20.0) {
			maximumClockDelay *= 2.0;
		}
	}
	
	public void toggleIsPaused() {
		isPaused = !isPaused;
	}
	
	public boolean getIsPaused() {
		return isPaused;
	}
	
	public void addMarker(Marker marker) {
		if (marker.getId() == 1) {
			marker1 = marker;
		}
		if (marker.getId() == 2) {
			marker2 = marker;
		}
	}
	
	public Marker getMarker(int which) {
		if (which == 1) {
			return marker1;
		}
		if (which == 2) {
			return marker2;
		}
		return null;
	}
	
	public boolean hasSelection() {
		return (marker1 != null && marker2 != null);
	}
	
	public Pos getSelectionTopLeft() {
		Pos tempPos1 = marker1.getPos();
		Pos tempPos2 = marker2.getPos();
		int tempPosX;
		int tempPosY;
		if (tempPos1.getX() < tempPos2.getX()) {
			tempPosX = tempPos1.getX();
		} else {
			tempPosX = tempPos2.getX();
		}
		if (tempPos1.getY() < tempPos2.getY()) {
			tempPosY = tempPos1.getY();
		} else {
			tempPosY = tempPos2.getY();
		}
		return new Pos(tempPosX, tempPosY);
	}
	
	public Pos getSelectionBottomRight() {
		Pos tempPos1 = marker1.getPos();
		Pos tempPos2 = marker2.getPos();
		int tempPosX;
		int tempPosY;
		if (tempPos1.getX() > tempPos2.getX()) {
			tempPosX = tempPos1.getX();
		} else {
			tempPosX = tempPos2.getX();
		}
		if (tempPos1.getY() > tempPos2.getY()) {
			tempPosY = tempPos1.getY();
		} else {
			tempPosY = tempPos2.getY();
		}
		return new Pos(tempPosX, tempPosY);
	}
	
	public Clipboard getClipboard() {
		return clipboard;
	}
	
	public void copySelectionToClipboard() {
		if (!hasSelection()) {
			return;
		}
		Pos tempPos1 = getSelectionTopLeft();
		Pos tempPos2 = getSelectionBottomRight();
		int tempWidth = tempPos2.getX() - tempPos1.getX() + 1;
		int tempHeight = tempPos2.getY() - tempPos1.getY() + 1;
		clipboard.setDimensions(tempWidth, tempHeight);
		Pos tempOffset;
		tempOffset = marker1.getPos().copy();
		tempOffset.subtract(tempPos1);
		clipboard.setMarker1Offset(tempOffset);
		Pos tempPos = new Pos(0, 0);
		tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < tempHeight) {
			tempPos = tempPos1.copy();
			tempPos.add(tempOffset);
			Block tempBlock = getBlock(tempPos).copy();
			clipboard.setBlock(tempOffset, tempBlock);
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= tempWidth) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
	}
	
	public void cutSelectionToClipboard() {
		if (!hasSelection()) {
			return;
		}
		Pos tempPos1 = getSelectionTopLeft();
		Pos tempPos2 = getSelectionBottomRight();
		int tempWidth = tempPos2.getX() - tempPos1.getX() + 1;
		int tempHeight = tempPos2.getY() - tempPos1.getY() + 1;
		clipboard.setDimensions(tempWidth, tempHeight);
		Pos tempOffset;
		tempOffset = marker1.getPos().copy();
		tempOffset.subtract(tempPos1);
		clipboard.setMarker1Offset(tempOffset);
		Pos tempPos = new Pos(0, 0);
		tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < tempHeight) {
			tempPos = tempPos1.copy();
			tempPos.add(tempOffset);
			Block tempBlock = getBlock(tempPos).copy();
			setBlock(tempPos, empty);
			clipboard.setBlock(tempOffset, tempBlock);
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= tempWidth) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
		userHasChangedBlock = true;
		handleUserChanges();
	}
	
	public void pasteClipboard() {
		if (marker1 == null) {
			return;
		}
		Pos tempPos = new Pos(0, 0);
		Pos tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < clipboard.getHeight()) {
			Block tempBlock = clipboard.getBlock(tempOffset).copy();
			if (!(tempBlock instanceof Empty)) {
				tempPos = marker1.getPos().copy();
				tempPos.subtract(clipboard.getMarker1Offset());
				tempPos.add(tempOffset);
				setBlock(tempPos, tempBlock);
			}
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= clipboard.getWidth()) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
		userHasChangedBlock = true;
		handleUserChanges();
	}
	
	public void rotateClipboardClockwise() {
		clipboard.rotateClockwise();
	}
	
	public void mirrorClipboardHorizontally() {
		clipboard.mirrorHorizontally();
	}
	
	public void fillSelection() {
		if (!hasSelection()) {
			return;
		}
		Pos tempPos1 = getSelectionTopLeft();
		Pos tempPos2 = getSelectionBottomRight();
		int tempWidth = tempPos2.getX() - tempPos1.getX() + 1;
		int tempHeight = tempPos2.getY() - tempPos1.getY() + 1;
		Pos tempPos = new Pos(0, 0);
		Pos tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < tempHeight) {
			tempPos = tempPos1.copy();
			tempPos.add(tempOffset);
			brush.addToWorld(this, tempPos);
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= tempWidth) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
		userHasChangedBlock = true;
		handleUserChanges();
	}
	
	public String getSaveCode() {
		List<String> tempEntryList = new ArrayList<String>();
		int index = 0;
		while (index < chunkList.size()) {
			Chunk tempChunk = chunkList.get(index);
			addChunkSaveCodeToList(tempChunk, tempEntryList);
			index += 1;
		}
		return String.join(";", tempEntryList);
	}
	
	public void addChunkSaveCodeToList(Chunk chunk, List<String> list) {
		if (chunk.isEmpty()) {
			return;
		}
		Pos tempPos = new Pos(0, 0);
		Pos tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < Chunk.size) {
			if (!chunk.rowIsEmpty(tempOffset.getY())) {
				List<Integer> tempList = new ArrayList<Integer>();
				tempList.add(saveCodePosCommand);
				tempList.add(chunk.getPos().getX());
				tempList.add(chunk.getPos().getY() + tempOffset.getY());
				List<String> tempList2 = convertIntListToStringList(tempList);
				list.add(String.join(",", tempList2));
				int tempEmptyCount = 0;
				tempOffset.setX(0);
				while (tempOffset.getX() < Chunk.size) {
					tempPos.set(chunk.getPos());
					tempPos.add(tempOffset);
					Block tempBlock = chunk.getBlock(tempPos);
					if (tempBlock == null) {
						tempEmptyCount += 1;
					} else {
						if (tempBlock instanceof Empty) {
							tempEmptyCount += 1;
						} else {
							if (tempEmptyCount > 0) {
								list.add(convertEmptyToSaveCode(tempEmptyCount));
								tempEmptyCount = 0;
							}
							list.add(convertBlockToSaveCode(tempBlock));
						}
					}
					tempOffset.setX(tempOffset.getX() + 1);
				}
				if (tempEmptyCount > 0) {
					list.add(convertEmptyToSaveCode(tempEmptyCount));
					tempEmptyCount = 0;
				}
			}
			tempOffset.setY(tempOffset.getY() + 1);
		}
	}
	
	public int getBlockSaveCodeIndex(Block block) {
		int index = 0;
		while (index < saveCodeBlockList.size()) {
			Block tempBlock = saveCodeBlockList.get(index);
			if (block.getClass().equals(tempBlock.getClass())) {
				return index;
			}
			index += 1;
		}
		return -1;
	}
	
	public List<String> convertIntListToStringList(List<Integer> list) {
		List<String> output = new ArrayList<String>();
		int index = 0;
		while (index < list.size()) {
			int tempNumber = list.get(index);
			output.add(Integer.toString(tempNumber));
			index += 1;
		}
		return output;
	}
	
	public List<Integer> convertStringListToIntList(String[] list) {
		List<Integer> output = new ArrayList<Integer>();
		int index = 0;
		while (index < list.length) {
			String tempText = list[index];
			output.add(Integer.parseInt(tempText));
			index += 1;
		}
		return output;
	}
	
	public String convertBlockToSaveCode(Block block) {
		int tempBlockIndex = getBlockSaveCodeIndex(block);
		List<Integer> tempList = new ArrayList<Integer>();
		tempList.add(tempBlockIndex);
		block.addSaveDataToList(tempList);
		List<String> tempList2 = convertIntListToStringList(tempList);
		return String.join(",", tempList2);
	}
	
	public String convertEmptyToSaveCode(int amount) {
		int tempBlockIndex = getBlockSaveCodeIndex(empty);
		List<Integer> tempList = new ArrayList<Integer>();
		tempList.add(tempBlockIndex);
		tempList.add(amount);
		List<String> tempList2 = convertIntListToStringList(tempList);
		return String.join(",", tempList2);
	}
	
	public void setSaveCode(String code) {
		clearMap();
		if (code.length() < 1) {
			return;
		}
		String[] tempEntryList = code.split(";"); 
		Pos tempPos = new Pos(0, 0);
		int index = 0;
		while (index < tempEntryList.length) {
			String tempEntry = tempEntryList[index];
			List<Integer> tempList = convertStringListToIntList(tempEntry.split(","));
			int tempCommand = tempList.get(0);
			if (tempCommand == saveCodePosCommand) {
				tempPos.setX(tempList.get(1));
				tempPos.setY(tempList.get(2));
			} else {
				Block tempBlock = saveCodeBlockList.get(tempCommand);
				if (tempBlock instanceof Empty) {
					tempPos.setX(tempPos.getX() + tempList.get(1));
				} else {
					tempBlock.setSaveDataFromList(tempList, 1);
					tempBlock.addToWorld(this, tempPos);
					tempPos.setX(tempPos.getX() + 1);
				}
			}
			index += 1;
		}
	}
	
	public void setUserHasChangedBlock(boolean value) {
		userHasChangedBlock = value;
	}
	
	public void handleUserChanges() {
		if (userHasChangedBlock) {
			while (changeHistoryIndex < changeHistory.size() - 1) {
				changeHistory.remove(changeHistory.size() - 1);
			}
			changeHistory.add(getSaveCode());
			changeHistoryIndex += 1;
			while (changeHistory.size() > 30) {
				changeHistory.remove(0);
				changeHistoryIndex -= 1;
			}
			userHasChangedBlock = false;
			setSaveHasBeenChanged(true);
		}
	}
	
	public void undoUserChange() {
		if (changeHistoryIndex > 0) {
			changeHistoryIndex -= 1;
			String tempSaveCode = changeHistory.get(changeHistoryIndex);
			setSaveCode(tempSaveCode);
			if (changeHistoryIndex == 0) {
				setSaveHasBeenChanged(false);
			}
		}
	}
	
	public void redoUserChange() {
		if (changeHistoryIndex < changeHistory.size() - 1) {
			changeHistoryIndex += 1;
			String tempSaveCode = changeHistory.get(changeHistoryIndex);
			setSaveCode(tempSaveCode);
			setSaveHasBeenChanged(true);
		}
	}
	
	public File getFile() {
		return file;
	}
	
	public void setFile(File inputFile) {
		file = inputFile;
	}
	
	public void saveToFile() {
		try {
			String tempSaveCode = getSaveCode();
			byte[] tempData = tempSaveCode.getBytes("UTF-8");
			FileOutputStream tempStream = new FileOutputStream(file);
            tempStream.write(tempData);
            tempStream.close();
        } catch(FileNotFoundException ex) {
        	System.out.println("Unable to find file.");              
        } catch(IOException ex) {
            System.out.println("Error while reading file.");
        }
		setSaveHasBeenChanged(false);
	}
	
	public void openFromFile() {
		clearChangeHistory();
		try {
			FileInputStream tempStream = new FileInputStream(file);
            byte[] tempData = new byte[(int)file.length()];
            tempStream.read(tempData);
            tempStream.close();
            String tempSaveCode = new String(tempData, "UTF-8");
            setSaveCode(tempSaveCode);
        } catch(FileNotFoundException ex) {
        	System.out.println("Unable to find file.");              
        } catch(IOException ex) {
            System.out.println("Error while reading file.");
        }
		userHasChangedBlock = true;
		handleUserChanges();
		setSaveHasBeenChanged(false);
	}
	
	public void setSaveHasBeenChanged(boolean value) {
		saveHasBeenChanged = value;
		int index = 0;
		while (index < saveChangeSubscriberList.size()) {
			SaveChangeSubscriber tempSubscriber = saveChangeSubscriberList.get(index);
			tempSubscriber.saveChangeEvent();
			index += 1;
		}
	}
	
	public boolean getSaveHasBeenChanged() {
		return saveHasBeenChanged;
	}
	
	public void addSaveChangeSubscriber(SaveChangeSubscriber subscriber) {
		saveChangeSubscriberList.add(subscriber);
	}
	
}
