package logicland2.block;

import java.util.List;

import logicland2.Brushable;
import logicland2.Pos;
import logicland2.Signal;
import logicland2.World;

public abstract class Block implements Brushable {

	public abstract Block copy();
	
	// Return true if a change has occurred.
	public boolean addToWorld(World world, Pos pos) {
		Block tempBlock = world.getBlock(pos);
		if (shouldPaintOver(tempBlock)) {
			world.setBlock(pos, copy());
			return true;
		} else {
			return false;
		}
	}
	
	public void addEvent(World world, Pos pos) {
		// Do nothing.
	}
	
	public void removeEvent(World world, Pos pos) {
		// Do nothing.
	}
	
	public abstract boolean isWireConnectable();
	
	// Sides:
	// 0 = top
	// 1 = right
	// 2 = bottom
	// 3 = left
	public abstract boolean connectsOnSide(int side);
	
	public void propagateSignal(World world, Signal signal) {
		// Do nothing.
	}
	
	public void populateCopy(Block block) {
		// Do nothing.
	}
	
	public void rotateClockwise() {
		// Do nothing.
	}
	
	public void mirrorHorizontally() {
		// Do nothing.
	}
	
	// Return false if the blocks are structurally and functionally equivalent.
	public boolean shouldPaintOver(Block block) {
		return !this.getClass().equals(block.getClass());
	}
	
	public void addSaveDataToList(List<Integer> list) {
		// Do nothing.
	}
	
	public void setSaveDataFromList(List<Integer> list, int offset) {
		// Do nothing.
	}
	
	@Override
	public boolean canOverlapBlock() {
		return false;
	}
	
}
