package logicland2.block;

import java.awt.Color;
import java.awt.Graphics;
import java.util.List;

import logicland2.CustomPanel;
import logicland2.Pos;
import logicland2.Signal;
import logicland2.World;

public abstract class LogicInput extends ModuleConnectableBlock {

	private static Color color = new Color(0, 192, 0);
	
	private boolean state = false;
	
	@Override
	public void draw(CustomPanel panel, Graphics graphics, Pos pos) {
		Pos tempPos = pos.copy();
		tempPos.subtract(panel.getCameraPos());
		tempPos.scale(panel.getZoom());
		if (panel.getZoom() >= 8.0) {
			panel.drawSprite(graphics, tempPos, 48);
		} else {
			graphics.setColor(color);
			int tempSize = (int)(panel.getZoom());
			if (tempSize < 1) {
				tempSize = 1;
			}
			graphics.fillRect(tempPos.getX(), tempPos.getY(), tempSize, tempSize);
		}
	}

	@Override
	public boolean isWireConnectable() {
		return true;
	}

	@Override
	public boolean connectsOnSide(int side) {
		return true;
	}
	
	public boolean getState() {
		return state;
	}
	
	public void setState(boolean value) {
		state = value;
	}
	
	public void setStateBySignalValue(boolean value) {
		state = value;
	}
	
	@Override
	public boolean updateConnection(World world, Pos pos) {
		int tempSide = 0;
		while (tempSide <= 3) {
			Block tempBlock = world.getBlock(World.getPosOnSide(pos, tempSide));
			if (tempBlock instanceof ModuleBlock) {
				ModuleBlock tempModuleBlock = (ModuleBlock)tempBlock;
				if (setBaseModuleBlock(tempModuleBlock.getBaseModuleBlock())) {
					return true;
				}
			}
			tempSide += 1;
		}
		return false;
	}
	
	@Override
	public void propagateSignal(World world, Signal signal) {
		setStateBySignalValue(signal.getValue());
		if (getBaseModuleBlock() != null) {
			getBaseModuleBlock().receiveUpdate(this);
		}
	}
	
	@Override
	public void populateCopy(Block block) {
		LogicInput tempBlock = (LogicInput)block;
		tempBlock.setState(state);
	}
	
	@Override
	public void addSaveDataToList(List<Integer> list) {
		if (state) {
			list.add(1);
		} else {
			list.add(0);
		}
	}
	
	@Override
	public void setSaveDataFromList(List<Integer> list, int offset) {
		int tempNumber = list.get(offset + 0);
		state = (tempNumber != 0);
	}
	
}
