package logicland2.block;

import java.awt.Color;
import java.awt.Graphics;

import logicland2.CustomPanel;
import logicland2.Pos;
import logicland2.Signal;
import logicland2.World;

public abstract class LogicOutput extends ModuleConnectableBlock {

	private static Color color = new Color(255, 0, 0);
	
	private boolean state = false;
	private Pos pos;
	private World world;
	
	@Override
	public void addEvent(World inputWorld, Pos inputPos) {
		pos = inputPos.copy();
		world = inputWorld;
		super.addEvent(world, inputPos);
	}
	
	@Override
	public void draw(CustomPanel panel, Graphics graphics, Pos pos) {
		Pos tempPos = pos.copy();
		tempPos.subtract(panel.getCameraPos());
		tempPos.scale(panel.getZoom());
		if (panel.getZoom() >= 8.0) {
			panel.drawSprite(graphics, tempPos, 49);
		} else {
			graphics.setColor(color);
			int tempSize = (int)(panel.getZoom());
			if (tempSize < 1) {
				tempSize = 1;
			}
			graphics.fillRect(tempPos.getX(), tempPos.getY(), tempSize, tempSize);
		}
	}

	@Override
	public boolean isWireConnectable() {
		return true;
	}

	@Override
	public boolean connectsOnSide(int side) {
		return true;
	}
	
	public boolean getState() {
		return state;
	}
	
	public void setState(boolean value) {
		state = value;
	}
	
	public Pos getPos() {
		return pos;
	}
	
	public World getWorld() {
		return world;
	}
	
	@Override
	public boolean updateConnection(World world, Pos pos) {
		int tempSide = 0;
		while (tempSide <= 3) {
			Block tempBlock = world.getBlock(World.getPosOnSide(pos, tempSide));
			if (tempBlock instanceof ModuleBlock) {
				ModuleBlock tempModuleBlock = (ModuleBlock)tempBlock;
				if (setBaseModuleBlock(tempModuleBlock.getBaseModuleBlock())) {
					return true;
				}
			}
			tempSide += 1;
		}
		return false;
	}
	
	public void dataChangeEvent(boolean value) {
		// Do nothing.
	}
	
	public void carryDataChangeEvent(boolean value) {
		// Do nothing.
	}
	
	public void sendSignal(boolean value) {
		int tempSide = 0;
		while (tempSide <= 3) {
			Pos tempPos = World.getPosOnSide(getPos(), tempSide);
			world.addSignal(new Signal(tempPos, (tempSide + 2) % 4, value));
			tempSide += 1;
		}
	}
	
}
