package logicland2.block;

import java.util.ArrayList;
import java.util.List;

public class ModuleCore {
	
	private List<LogicInput> logicInputList = new ArrayList<LogicInput>();
	private List<LogicOutput> logicOutputList = new ArrayList<LogicOutput>();
	private List<ModuleBlock> moduleBlockList = new ArrayList<ModuleBlock>();
	private boolean data = false;
	private boolean carryData = false;

	public List<LogicInput> getLogicInputs() {
		return logicInputList;
	}
	
	public List<LogicOutput> getLogicOutputs() {
		return logicOutputList;
	}
	
	public List<ModuleBlock> getModuleBlocks() {
		return moduleBlockList;
	}
	
	public void addLogicInput(LogicInput block) {
		logicInputList.add(block);
	}
	
	public void addLogicOutput(LogicOutput block) {
		logicOutputList.add(block);
	}
	
	public void addModuleBlock(ModuleBlock block) {
		moduleBlockList.add(block);
	}
	
	public void removeLogicInput(LogicInput block) {
		logicInputList.remove(block);
	}
	
	public void removeLogicOutput(LogicOutput block) {
		logicOutputList.remove(block);
	}
	
	public void removeModuleBlock(ModuleBlock block) {
		moduleBlockList.remove(block);
	}
	
	public void addBlock(ModuleConnectableBlock block) {
		if (block instanceof LogicInput) {
			addLogicInput((LogicInput)block);
		}
		if (block instanceof LogicOutput) {
			addLogicOutput((LogicOutput)block);
		}
		if (block instanceof ModuleBlock) {
			addModuleBlock((ModuleBlock)block);
		}
	}
	
	public void removeBlock(ModuleConnectableBlock block) {
		if (block instanceof LogicInput) {
			removeLogicInput((LogicInput)block);
		}
		if (block instanceof LogicOutput) {
			removeLogicOutput((LogicOutput)block);
		}
		if (block instanceof ModuleBlock) {
			removeModuleBlock((ModuleBlock)block);
		}
	}
	
	public boolean getData() {
		return data;
	}
	
	public void setData(boolean inputData) {
		if (data != inputData) {
			data = inputData;
			int index = 0;
			while (index < logicOutputList.size()) {
				LogicOutput tempBlock = logicOutputList.get(index);
				tempBlock.dataChangeEvent(data);
				index += 1;
			}
		}
	}
	
	public boolean getCarryData() {
		return data;
	}
	
	public void setCarryData(boolean inputData) {
		if (carryData != inputData) {
			carryData = inputData;
			int index = 0;
			while (index < logicOutputList.size()) {
				LogicOutput tempBlock = logicOutputList.get(index);
				tempBlock.carryDataChangeEvent(carryData);
				index += 1;
			}
		}
	}
	
	public int getDataLogicInputCount() {
		int output = 0;
		int index = 0;
		while (index < logicInputList.size()) {
			LogicInput tempBlock = logicInputList.get(index);
			if (tempBlock instanceof DataLogicInput) {
				output += 1;
			}
			index += 1;
		}
		return output;
	}
	
	public int getActiveDataLogicInputCount() {
		int output = 0;
		int index = 0;
		while (index < logicInputList.size()) {
			LogicInput tempBlock = logicInputList.get(index);
			if (tempBlock instanceof DataLogicInput && tempBlock.getState()) {
				output += 1;
			}
			index += 1;
		}
		return output;
	}
	
	public void populateCopyData(ModuleCore core) {
		core.setData(data);
		core.setCarryData(carryData);
	}
	
	public int getSaveData() {
		int output = 0;
		if (data) {
			output += 1;
		}
		if (carryData) {
			output += 2;
		}
		return output;
	}
	
	public void setSaveData(int value) {
		data = (value % 2 == 1);
		carryData = (value % 4 > 1);
	}
	
}
