package logicland2.block;

import java.awt.Color;
import java.awt.Graphics;
import java.util.List;

import logicland2.Brushable;
import logicland2.CustomPanel;
import logicland2.Pos;
import logicland2.Signal;
import logicland2.World;

public class Wire extends Block {

	private static Color activeColor = new Color(255, 255, 255);
	private static Color inactiveColor = new Color(0, 0, 0);
	
	// 0 through 1 = Straight wire.
	// 2 through 5 = Bent wire.
	// 6 through 9 = Three split wire.
	// 10 = Four split wire.
	// 11 = Cross over.
	private int direction;
	private boolean state1 = false;
	private boolean state2 = false;
	
	public Wire(int inputDirection) {
		direction = inputDirection;
	}
	
	@Override
	public Block copy() {
		Block output = new Wire(direction);
		populateCopy(output);
		return output;
	}
	
	public int getDirection() {
		return direction;
	}
	
	public void setState1(boolean value) {
		state1 = value;
	}
	
	public void setState2(boolean value) {
		state2 = value;
	}

	@Override
	public void draw(CustomPanel panel, Graphics graphics, Pos pos) {
		Pos tempPos = pos.copy();
		tempPos.subtract(panel.getCameraPos());
		tempPos.scale(panel.getZoom());
		if (panel.getZoom() >= 8.0) {
			int tempWhich = 16 + direction;
			if (state1) {
				tempWhich += 16;
			}
			if (state2) {
				tempWhich += 1;
			}
			panel.drawSprite(graphics, tempPos, tempWhich);
		} else {
			if (state1 || state2) {
				graphics.setColor(activeColor);
			} else {
				graphics.setColor(inactiveColor);
			}
			int tempSize = (int)(panel.getZoom());
			if (tempSize < 1) {
				tempSize = 1;
			}
			graphics.fillRect(tempPos.getX(), tempPos.getY(), tempSize, tempSize);
		}
	}

	@Override
	public Brushable getNextVariation() {
		if (direction == 0) {
			direction += 1;
			return this;
		}
		if (direction == 1) {
			direction = 0;
			return this;
		}
		if (direction >= 2 && direction < 5) {
			direction += 1;
			return this;
		}
		if (direction == 5) {
			direction = 2;
			return this;
		}
		if (direction >= 6 && direction < 9) {
			direction += 1;
			return this;
		}
		if (direction == 9) {
			direction = 6;
			return this;
		}
		return this;
	}

	@Override
	public Brushable getPreviousVariation() {
		if (direction == 1) {
			direction -= 1;
			return this;
		}
		if (direction == 0) {
			direction = 1;
			return this;
		}
		if (direction > 2 && direction <= 5) {
			direction -= 1;
			return this;
		}
		if (direction == 2) {
			direction = 5;
			return this;
		}
		if (direction > 6 && direction <= 9) {
			direction -= 1;
			return this;
		}
		if (direction == 6) {
			direction = 9;
			return this;
		}
		return this;
	}
	
	@Override
	public boolean isWireConnectable() {
		return true;
	}
	
	@Override
	public boolean connectsOnSide(int side) {
		if (direction == 0) {
			return (side == 0 || side == 2);
		}
		if (direction == 1) {
			return (side == 1 || side == 3);
		}
		if (direction == 2) {
			return (side == 0 || side == 1);
		}
		if (direction == 3) {
			return (side == 1 || side == 2);
		}
		if (direction == 4) {
			return (side == 2 || side == 3);
		}
		if (direction == 5) {
			return (side == 3 || side == 0);
		}
		if (direction == 6) {
			return (side != 3);
		}
		if (direction == 7) {
			return (side != 0);
		}
		if (direction == 8) {
			return (side != 1);
		}
		if (direction == 9) {
			return (side != 2);
		}
		if (direction == 10 || direction == 11) {
			return true;
		}
		return true;
	}
	
	public void connectOnSide(int side) {
		if (direction == 0) {
			if (side == 1) {
				direction = 6;
			}
			if (side == 3) {
				direction = 8;
			}
		} else if (direction == 1) {
			if (side == 0) {
				direction = 9;
			}
			if (side == 2) {
				direction = 7;
			}
		} else if (direction == 2) {
			if (side == 2) {
				direction = 6;
			}
			if (side == 3) {
				direction = 9;
			}
		} else if (direction == 3) {
			if (side == 3) {
				direction = 7;
			}
			if (side == 0) {
				direction = 6;
			}
		} else if (direction == 4) {
			if (side == 0) {
				direction = 8;
			}
			if (side == 1) {
				direction = 7;
			}
		} else if (direction == 5) {
			if (side == 1) {
				direction = 9;
			}
			if (side == 2) {
				direction = 8;
			}
		} else if (direction == 6) {
			if (side == 3) {
				direction = 11;
			}
		} else if (direction == 7) {
			if (side == 0) {
				direction = 11;
			}
		} else if (direction == 8) {
			if (side == 1) {
				direction = 11;
			}
		} else if (direction == 9) {
			if (side == 2) {
				direction = 11;
			}
		}
	}
	
	public void disconnectOnSide(int side) {
		if (direction == 2) {
			if (side == 0) {
				direction = 1;
			}
			if (side == 1) {
				direction = 0;
			}
		} else if (direction == 3) {
			if (side == 1) {
				direction = 0;
			}
			if (side == 2) {
				direction = 1;
			}
		} else if (direction == 4) {
			if (side == 2) {
				direction = 1;
			}
			if (side == 3) {
				direction = 0;
			}
		} else if (direction == 5) {
			if (side == 3) {
				direction = 0;
			}
			if (side == 0) {
				direction = 1;
			}
		} else if (direction == 6) {
			if (side == 0) {
				direction = 3;
			}
			if (side == 1) {
				direction = 0;
			}
			if (side == 2) {
				direction = 2;
			}
		} else if (direction == 7) {
			if (side == 1) {
				direction = 4;
			}
			if (side == 2) {
				direction = 1;
			}
			if (side == 3) {
				direction = 3;
			}
		} else if (direction == 8) {
			if (side == 2) {
				direction = 5;
			}
			if (side == 3) {
				direction = 0;
			}
			if (side == 0) {
				direction = 4;
			}
		} else if (direction == 9) {
			if (side == 3) {
				direction = 2;
			}
			if (side == 0) {
				direction = 1;
			}
			if (side == 1) {
				direction = 5;
			}
		} else if (direction == 10 || direction == 11) {
			if (side == 0) {
				direction = 7;
			}
			if (side == 1) {
				direction = 8;
			}
			if (side == 2) {
				direction = 9;
			}
			if (side == 3) {
				direction = 6;
			}
		}
	}

	@Override
	public void propagateSignal(World world, Signal signal) {
		if (direction == 11) {
			if (signal.getSide() == 0) {
				if (signal.getValue() == state1) {
					return;
				}
				Pos tempPos = World.getPosOnSide(signal.getPos(), 2);
				world.addSignal(new Signal(tempPos, 0, signal.getValue()));
				state1 = signal.getValue();
			}
			if (signal.getSide() == 1) {
				if (signal.getValue() == state2) {
					return;
				}
				Pos tempPos = World.getPosOnSide(signal.getPos(), 3);
				world.addSignal(new Signal(tempPos, 1, signal.getValue()));
				state2 = signal.getValue();
			}
			if (signal.getSide() == 2) {
				if (signal.getValue() == state1) {
					return;
				}
				Pos tempPos = World.getPosOnSide(signal.getPos(), 0);
				world.addSignal(new Signal(tempPos, 2, signal.getValue()));
				state1 = signal.getValue();
			}
			if (signal.getSide() == 3) {
				if (signal.getValue() == state2) {
					return;
				}
				Pos tempPos = World.getPosOnSide(signal.getPos(), 1);
				world.addSignal(new Signal(tempPos, 3, signal.getValue()));
				state2 = signal.getValue();
			}
		} else {
			if (signal.getValue() == state1) {
				return;
			}
			if (!connectsOnSide(signal.getSide())) {
				return;
			}
			int tempSide = 0;
			while (tempSide <= 3) {
				if (tempSide != signal.getSide() && connectsOnSide(tempSide)) {
					Pos tempPos = World.getPosOnSide(signal.getPos(), tempSide);
					world.addSignal(new Signal(tempPos, (tempSide + 2) % 4, signal.getValue()));
				}
				tempSide += 1;
			}
			state1 = signal.getValue();
		}
	}
	
	public void populateCopy(Block block) {
		Wire tempBlock = (Wire)block;
		tempBlock.setState1(state1);
		tempBlock.setState2(state2);
	}
	
	@Override
	public void rotateClockwise() {
		if (direction == 0) {
			direction += 1;
		} else if (direction == 1) {
			direction = 0;
		} else if (direction >= 2 && direction < 5) {
			direction += 1;
		} else if (direction == 5) {
			direction = 2;
		} else if (direction >= 6 && direction < 9) {
			direction += 1;
		} else if (direction == 9) {
			direction = 6;
		} else if (direction == 11) {
			boolean tempState = state2;
			state2 = state1;
			state1 = tempState;
		}
	}
	
	@Override
	public void mirrorHorizontally() {
		if (direction == 2) {
			direction = 5;
		} else if (direction == 5) {
			direction = 2;
		} else if (direction == 3) {
			direction = 4;
		} else if (direction == 4) {
			direction = 3;
		} else if (direction == 6) {
			direction = 8;
		} else if (direction == 8) {
			direction = 6;
		}
	}
	
	@Override
	public boolean shouldPaintOver(Block block) {
		if (block instanceof Wire) {
			Wire tempBlock = (Wire)block;
			return (direction != tempBlock.getDirection());
		} else {
			return true;
		}
	}
	
	@Override
	public void addSaveDataToList(List<Integer> list) {
		list.add(direction);
		int tempNumber = 0;
		if (state1) {
			tempNumber += 1;
		}
		if (state2) {
			tempNumber += 2;
		}
		list.add(tempNumber);
	}
	
	@Override
	public void setSaveDataFromList(List<Integer> list, int offset) {
		direction = list.get(offset + 0);
		int tempNumber = list.get(offset + 1);
		state1 = (tempNumber % 2 == 1);
		state2 = (tempNumber % 4 > 1);
	}

}
